@@ -52,7 +52,7 @@ viviBartMatrix <- function(trees,
                            metric = "propMean",
                            metricError = "CV",
                            reorder = FALSE
-                          ){
+){
 
   if (!(metric %in% c("propMean", "count", "adjusted"))) {
     stop("metric must be \"propMean\", \"count\", or \"adjusted\"")
@@ -69,21 +69,21 @@ viviBartMatrix <- function(trees,
   viviDf <- viviBartInternal(trees)
 
   if(type == 'standard'){
-   viviMat <-  viviBartStd(trees = trees,
-                                 data = viviDf,
-                                 metric = metric,
-                                 reorder = reorder
-                                 )
+    viviMat <-  viviBartStd(trees = trees,
+                            data = viviDf,
+                            metric = metric,
+                            reorder = reorder
+    )
   }else if(type == 'vsup'){
     viviMat <- viviBartVSUP(trees = trees,
-                                 data = viviDf,
-                                 metricError = metricError,
-                                 metric = metric,
-                                 reorder = reorder)
+                            data = viviDf,
+                            metricError = metricError,
+                            metric = metric,
+                            reorder = reorder)
   }else if(type == 'quantiles'){
     viviMat <- viviBartQuantile(trees = trees,
-                                   data = viviDf,
-                                   reorder = reorder)
+                                data = viviDf,
+                                reorder = reorder)
   }
 
   return(viviMat)
@@ -96,64 +96,73 @@ viviBartMatrix <- function(trees,
 # -------------------------------------------------------------------------
 # VIVI dataframe ----------------------------------------------------------
 # -------------------------------------------------------------------------
-
-viviBartInternal <- function(trees){
-
+viviBartInternal <- function(trees) {
   # Vimps -------------------------------------------------------------------
 
   # get vimps
   vimps <- bartMan::vimpBart(trees, type = 'prop')
   vimpsVal <- bartMan::vimpBart(trees, type = 'val')
 
-
   vImp <- colMeans(vimps)
   vimpsVal <- colSums(vimpsVal)
 
-  # get uncertainty measures
-  vimpSD <- apply(vimps, 2, sd)
-  upperVimp  <- vImp + 1.96 * vimpSD/sqrt(trees$nMCMC)
-  lowerVimp  <- vImp - 1.96 * vimpSD/sqrt(trees$nMCMC)
-  SEvimp <- sapply(as.data.frame(vimps), function(x) sd(x)/sqrt(length(x)))
-  CVvimp <- vimpSD / vImp
-  CVvimp[is.nan(CVvimp)] <- 0
+  n_iterations <- nrow(vimps)
 
-  # get quantiles of proportions
-  vimp25 <- apply(vimps, 2, function(x) quantile(x, c(.25)))
-  vimp50 <- apply(vimps, 2, function(x) quantile(x, c(.50)))
-  vimp75 <- apply(vimps, 2, function(x) quantile(x, c(.75)))
+  if (n_iterations > 1) {
+    # get uncertainty measures
+    vimpSD <- apply(vimps, 2, sd)
+    upperVimp <- vImp + 1.96 * vimpSD / sqrt(n_iterations)
+    lowerVimp <- vImp - 1.96 * vimpSD / sqrt(n_iterations)
+    SEvimp <- sapply(as.data.frame(vimps), function(x) sd(x) / sqrt(length(x)))
+    CVvimp <- vimpSD / vImp
+    CVvimp[is.nan(CVvimp)] <- 0
+
+    # get quantiles of proportions
+    vimp25 <- apply(vimps, 2, function(x) quantile(x, c(.25)))
+    vimp50 <- apply(vimps, 2, function(x) quantile(x, c(.50)))
+    vimp75 <- apply(vimps, 2, function(x) quantile(x, c(.75)))
+  } else {
+    # handle single iteration case
+    vimpSD <- rep(sd(vimps), length(vImp))
+    upperVimp <- vImp
+    lowerVimp <- vImp
+    SEvimp <- rep(sd(vimps) / sqrt(length(vimps)), length(vImp))
+    CVvimp <- vimpSD / vImp
+    vimp25 <- vImp
+    vimp50 <- vImp
+    vimp75 <- vImp
+  }
 
   # put together in dataframe
   vimpData <- cbind(vimpsVal, vImp, vimpSD, CVvimp, SEvimp,
                     lowerVimp, upperVimp, vimp25, vimp50, vimp75)
 
-  # vimpData <- vimpData %>%
-  #   as.data.frame() %>%
-  #   tibble::rownames_to_column(var = "variable") %>%
-  #   rename(count = vimpsVal, propMean = vImp, SD = vimpSD,
-  #          CV = CVvimp,  SE = SEvimp,
-  #          lowerCI = lowerVimp, upperCI = upperVimp,
-  #          lowerQ = vimp25, median = vimp50, upperQ = vimp75)
-
-
   vDf <- vimpData %>%
     as.data.frame()
 
   vDf$variable <- row.names(vimpData)
-  vimpData <- vDf |>
+  vimpData <- vDf %>%
     rename(count = vimpsVal, propMean = vImp, SD = vimpSD,
-           CV = CVvimp,  SE = SEvimp,
+           CV = CVvimp, SE = SEvimp,
            lowerCI = lowerVimp, upperCI = upperVimp,
-           lowerQ = vimp25, median = vimp50, upperQ = vimp75) |>
+           lowerQ = vimp25, median = vimp50, upperQ = vimp75) %>%
     select(variable, count, propMean, SD, CV, SE, lowerCI, upperCI, lowerQ, median, upperQ)
 
-
-
-# VInts -------------------------------------------------------------------
-
+  # VInts -------------------------------------------------------------------
 
   df <- trees$structure
   nam <- trees$varName
 
+
+
+  # remap subset
+  # df <- df %>%
+  #   mutate(iteration_old = iteration) %>%
+  #   mutate(iteration = as.integer(factor(iteration, levels = unique(iteration))))
+
+
+
+
   # cycle through trees and create list of Vints
   mkTree <- function(x, pos = 1L) {
     var <- x[pos]
@@ -169,7 +178,6 @@ viviBartInternal <- function(trees){
     }
   }
 
-
   tabTree <- function(tree, sep = ":") {
     x <- rep.int(NA_character_, tree[[4L]])
     pos <- 1L
@@ -196,66 +204,102 @@ viviBartInternal <- function(trees){
     }
   }
 
+  # f <- function(x) tabTree(mkTree(x))
+  # L <- tapply(df[["var"]], df[c("treeNum", "iteration")], f, simplify = FALSE)
+  #
+  # g <- function(l) {
+  #   x <- unlist(unname(l)) # UNLIST IS NOT SEPERATING THEMINT ITERATIONS IF ITERATIONS ARE EQUAL!!
+  #   tapply(x, names(x), sum)
+  # }
+  #
+  # if (n_iterations > 1) {
+  #   listVint <- apply(L, 2L, g)
+  #   listVint <- listVint[lengths(listVint) > 0] # remove empty list element
+  # }
+  # Define the function to apply to each element
   f <- function(x) tabTree(mkTree(x))
+
+  # Apply the function to each combination of treeNum and iteration, resulting in a list
   L <- tapply(df[["var"]], df[c("treeNum", "iteration")], f, simplify = FALSE)
 
+  # Define the function to process each list element
   g <- function(l) {
     x <- unlist(unname(l))
     tapply(x, names(x), sum)
   }
 
-  listVint <- apply(L, 2L, g)
-  listVint <- listVint[lengths(listVint)>0] # remove empty list element
+  # Process the list if there are multiple iterations
+  if (n_iterations > 1) {
+    listVint <- lapply(seq_len(ncol(L)), function(i) {
+      g(L[, i])
+    })
 
+    # Remove empty list elements
+    listVint <- listVint[lengths(listVint) > 0]
+  } else {
+    # Handle the single iteration case
+    listVint <- list()
+    for (i in 1:length(L)) {
+      single_iter_data <- L[[i]]
+      if (length(single_iter_data) > 0) {
+        single_iter_data <- single_iter_data[!is.na(single_iter_data)]
+        if (length(single_iter_data) > 0) {
+          listVint[[as.character(i)]] <- tapply(single_iter_data, names(single_iter_data), sum)
+        }
+      }
+    }
+  }
 
-  # turn into df
-  dfVint <- as.matrix(bind_rows(listVint))
-  dfVint[is.na(dfVint)] <- 0
-
-
-  # create a matrix of all possible combinations
   nam <- trees$varName
   namDF <- expand.grid(nam, nam)
 
   newName <- NULL
-  for(i in 1:length(namDF$Var1)){
+  for (i in 1:length(namDF$Var1)) {
     newName[i] <- paste0(namDF$Var2[i], ":", namDF$Var1[i])
   }
 
-  allCombMat <- matrix(NA, nrow = trees$nMCMC, ncol = length(newName))
-  colnames(allCombMat) <- newName
-
-  # join actual values into matirx of all combinations
-  oIdx <- match(colnames(dfVint), colnames(allCombMat))
-
-  if(nrow(dfVint) != nrow(allCombMat)){
-    missingRows <-  nrow(allCombMat) - nrow(dfVint)
-    dfVint <- rbind(dfVint, matrix(data = 0, ncol=ncol(dfVint), nrow=missingRows))
+  # turn into df
+  if (length(listVint) > 0) {
+    dfVint <- as.matrix(bind_rows(listVint))
+    dfVint[is.na(dfVint)] <- 0
+  } else {
+    dfVint <- matrix(0, nrow = 1, ncol = length(newName))
+    colnames(dfVint) <- newName
   }
 
-  allCombMat[ ,oIdx] <- dfVint
+  # create a matrix of all possible combinations
+  allCombMat <- matrix(NA, nrow = max(n_iterations, nrow(dfVint)), ncol = length(newName))
+  colnames(allCombMat) <- newName
+
+  # join actual values into matrix of all combinations
+  oIdx <- match(colnames(dfVint), colnames(allCombMat))
+
+  if (nrow(dfVint) < nrow(allCombMat)) {
+    missingRows <- nrow(allCombMat) - nrow(dfVint)
+    dfVint <- rbind(dfVint, matrix(data = 0, ncol = ncol(dfVint), nrow = missingRows))
+  }
+
+  allCombMat[, oIdx] <- dfVint
   allCombMat[is.na(allCombMat)] <- 0
   dfVint <- allCombMat
 
   # reorder names to make symmetrical
-  #vintNames <- reshape2::melt(dfVint)
   vintNames <- utils::stack(as.data.frame(dfVint))
   colnames(vintNames) = c('value', 'Var2')
-  vintNames <- vintNames[,2:1]
+  vintNames <- vintNames[, 2:1]
 
   dfName <- data.frame(nam = unique(vintNames$Var2))
-
   dfName$nam <- as.character(dfName$nam)
   newNames <- dfName %>%
     mutate(nam = map(
-      strsplit(nam, ":", fixed = T),
+      strsplit(nam, ":", fixed = TRUE),
       ~ sort(.x) %>% trimws(.) %>% paste0(., collapse = ':')
     ))
 
   colnames(dfVint) <- newNames$nam
 
   # add symmetrical columns together
-  dfVint <- t(apply(dfVint, 1, \(x) stats::ave(x, names(x), FUN = sum)))
+  dfVint <- t(apply(dfVint, 1, function(x) stats::ave(x, names(x), FUN = sum)))
 
   # get proportions
   propMatVint <- proportions(dfVint, 1)
@@ -264,22 +308,14 @@ viviBartInternal <- function(trees){
   propMatVintMean <- colMeans(propMatVint)
 
   # turn into df
-  # dfProps <- reshape2::melt(propMatVintMean)
-  # dfProps$var <- names(propMatVintMean)
-  # colnames(dfProps) <- c( "props", "var")
   dfProps <- utils::stack(propMatVintMean)
   colnames(dfProps) = c('props', 'var')
   dfProps$var <- as.character(dfProps$var)
 
-
-
   # add counts
   countMean <- colMeans(dfVint)
 
   # turn into df
-  # dfCountMean <- reshape2::melt(countMean)
-  # dfCountMean$var <- names(propMatVintMean)
-  # colnames(dfCountMean) <- c( "count", "var")
   dfCountMean <- utils::stack(countMean)
   colnames(dfCountMean) = c('count', 'var')
   dfCountMean$var <- as.character(dfCountMean$var)
@@ -289,40 +325,40 @@ viviBartInternal <- function(trees){
     var = dfCountMean$var,
     count = dfCountMean$count,
     props = dfProps$props
-
   )
 
   # Get uncertainty metrics -------------------------------------------------
 
-  vintSD <- apply(propMatVint, 2, sd)
-  # vintSD <- vintSD  |>
-  #   reshape2::melt()  |>
-  #   mutate(var = names(propMatVintMean))
-  # colnames(vintSD) <- c( "SD", "var")
+  if (n_iterations > 1) {
+    vintSD <- apply(propMatVint, 2, sd)
+    vintSE <- vintSD / sqrt(n_iterations)
+
+    vint25 <- apply(propMatVint, 2, function(x) quantile(x, c(.25)))
+    vint50 <- apply(propMatVint, 2, function(x) quantile(x, c(.50)))
+    vint75 <- apply(propMatVint, 2, function(x) quantile(x, c(.75)))
+  } else {
+    vintSD <- rep(sd(propMatVintMean), length(propMatVintMean))
+    vintSE <- rep(sd(propMatVintMean) / sqrt(length(propMatVintMean)), length(propMatVintMean))
+    vint25 <- propMatVintMean
+    vint50 <- propMatVintMean
+    vint75 <- propMatVintMean
+  }
+
+  # Ensure the vectors have names
+  names(vintSD) <- names(propMatVintMean)
+  names(vintSE) <- names(propMatVintMean)
+  names(vint25) <- names(propMatVintMean)
+  names(vint50) <- names(propMatVintMean)
+  names(vint75) <- names(propMatVintMean)
+
   vintSD <- utils::stack(vintSD)
   colnames(vintSD) = c('SD', 'var')
   vintSD$var <- as.character(vintSD$var)
 
-  vintSE <- vintSD$SD/sqrt(trees$nMCMC)
-  names(vintSE) <- vintSD$var
-  # vintSE <- vintSE  |>
-  #   reshape2::melt()  |>
-  #   mutate(var = names(propMatVintMean))
-  # colnames(vintSE) <- c( "SE", "var")
   vintSE <- utils::stack(vintSE)
   colnames(vintSE) = c('SE', 'var')
   vintSE$var <- as.character(vintSE$var)
 
-
-  # get quantiles of proportions
-  vint25 <- apply(propMatVint, 2, function(x) quantile(x, c(.25)))
-  vint50 <- apply(propMatVint, 2, function(x) quantile(x, c(.50)))
-  vint75 <- apply(propMatVint, 2, function(x) quantile(x, c(.75)))
-
-  # vint25 <- vint25  |> reshape2::melt()  |> mutate(var = names(propMatVintMean))
-  # vint50 <- vint50  |> reshape2::melt()  |> mutate(var = names(propMatVintMean))
-  # vint75 <- vint75  |> reshape2::melt()  |> mutate(var = names(propMatVintMean))
-
   vint25 <- utils::stack(vint25)
   colnames(vint25) <- c('value', 'var')
   vint25$var <- as.character(vint25$var)
@@ -345,8 +381,8 @@ viviBartInternal <- function(trees){
     q75 = vint75$value
   )
 
-  dfPropCount$SD <-  errorDF$SD
-  dfPropCount$SE <-  errorDF$SE
+  dfPropCount$SD <- errorDF$SD
+  dfPropCount$SE <- errorDF$SE
   dfPropCount$Q25 <- errorDF$q25
   dfPropCount$Q50 <- errorDF$q50
   dfPropCount$Q75 <- errorDF$q75
@@ -365,20 +401,20 @@ viviBartInternal <- function(trees){
     select(var, count, propMean, SD, SE, Q25, Q50, Q75, -props) %>%
     distinct() %>%
     ungroup()
-  #dfFinal$var <- unlist(dfFinal$var)
 
   # add coeff of variance
   dfFinal$CV <- dfFinal$SD / dfFinal$propMean
+  dfFinal$CV[is.infinite(dfFinal$CV)] <- 0
   dfFinal$CV[is.nan(dfFinal$CV)] <- 0
 
   # reorder
-  dfFinal <-  dfFinal |> select(var, count, propMean, SD, CV, SE, Q25, Q50, Q75)
+  dfFinal <- dfFinal %>% select(var, count, propMean, SD, CV, SE, Q25, Q50, Q75)
 
   # add adjustment
   vimpsAdj <- bartMan::vimpBart(trees, type = 'propMean')
 
   splitN <- t(simplify2array(strsplit(as.character(dfFinal[["var"]]), ":")))
-  values <- t(apply(splitN, 1, function(x){c(vimpsAdj[x[1]], vimpsAdj[x[2]])}))
+  values <- t(apply(splitN, 1, function(x) { c(vimpsAdj[x[1]], vimpsAdj[x[2]]) }))
   suppressMessages(
     res <- cbind(dfFinal, values)
   )
@@ -386,23 +422,22 @@ viviBartInternal <- function(trees){
                   'SE', 'lowerQ', 'median', 'upperQ',
                   'propVimp1', 'propVimp2')
 
-  trans <- apply(res[,c('propMean', 'propVimp1', 'propVimp2')], 1, function(x){
-    (x[1]-x[2]*x[3])/sqrt(x[2]*x[3])
+  trans <- apply(res[, c('propMean', 'propVimp1', 'propVimp2')], 1, function(x) {
+    (x[1] - x[2] * x[3]) / sqrt(x[2] * x[3])
   })
 
   dfFinal$adjusted <- trans
-  dfFinal$adjusted[dfFinal$adjusted <=  0] <- 0
-  dfFinal$adjusted[is.na( dfFinal$adjusted)] <- 0
+  dfFinal$adjusted[dfFinal$adjusted <= 0] <- 0
+  dfFinal$adjusted[is.na(dfFinal$adjusted)] <- 0
 
   names(dfFinal) <- c('var', 'count', 'propMean', 'SD', "CV",
-                      'SE',  'lowerQ', 'median',
+                      'SE', 'lowerQ', 'median',
                       'upperQ', 'adjusted')
 
   myList <- list(Vimp = vimpData, Vint = dfFinal)
 
   return(myList)
-
-}
+  }
 
 
 
@@ -416,7 +451,7 @@ viviBartStd <- function(trees,
                         data,
                         reorder = FALSE,
                         metric = "propMean"
-                       ){
+){
 
   propFinal <- data$Vint
   vars2  <- t(simplify2array(strsplit(as.character(propFinal[["var"]]), ":")))
@@ -506,6 +541,7 @@ viviBartVSUP <- function(trees,
 }
 
 
+
 # -------------------------------------------------------------------------
 # Quantile matrix --------------------------------------------------------
 # -------------------------------------------------------------------------
